#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2017 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# \package nxsconfigtool nexdatas
# \file DataSources.py
# Data Sources for  datasource dialog class

""" widget for different types of datasources """

from PyQt5.QtCore import (Qt, )
from PyQt5.QtWidgets import (QMessageBox, QTableWidgetItem)
from PyQt5.QtXml import (QDomDocument)
from PyQt5 import uic
import os
import sys

# from .ui.ui_clientdsdlg import Ui_ClientDsDlg
# from .ui.ui_dbdsdlg import Ui_DBDsDlg
# from .ui.ui_tangodsdlg import Ui_TangoDsDlg
# from .ui.ui_pyevaldsdlg import Ui_PyEvalDsDlg

from .DomTools import DomTools

_clientformclass, _clientbaseclass = uic.loadUiType(
    os.path.join(os.path.dirname(os.path.abspath(__file__)),
                 "ui", "clientdsdlg.ui"))

_dbformclass, _dbbaseclass = uic.loadUiType(
    os.path.join(os.path.dirname(os.path.abspath(__file__)),
                 "ui", "dbdsdlg.ui"))

_tangoformclass, _tangobaseclass = uic.loadUiType(
    os.path.join(os.path.dirname(os.path.abspath(__file__)),
                 "ui", "tangodsdlg.ui"))

_pyevalformclass, _pyevalbaseclass = uic.loadUiType(
    os.path.join(os.path.dirname(os.path.abspath(__file__)),
                 "ui", "pyevaldsdlg.ui"))

if sys.version_info > (3,):
    unicode = str


# CLIENT dialog impementation
class ClientSource(object):
    # allowed subitems
    subItems = ["record", "doc"]
    # variables
    var = {
        # client record name
        "recordName": u''}

    # constructor
    # \param main datasource dialog
    def __init__(self, main):
        # widget class
        self.widgetClass = _clientformclass
        # widget
        self.ui = None
        #  main datasource dialog
        self.main = main

    # checks if widget button should be enable
    # \returns if widget button should be enable
    def isEnable(self):
        return bool(self.ui.cRecNameLineEdit.text())

    # calls updateUi when the name text is changing
    def __cRecNameLineEdit(self):
        combo = unicode(self.main.ui.typeComboBox.currentText())
        self.main.updateUi(combo)

    # connects the dialog actions
    def connectWidgets(self):
        try:
            self.ui.cRecNameLineEdit.textChanged.disconnect(
                self.__cRecNameLineEdit)
        except Exception:
            pass
        self.ui.cRecNameLineEdit.textChanged.connect(
            self.__cRecNameLineEdit)
        # self.main.disconnect(
        #     self.ui.cRecNameLineEdit, SIGNAL("textChanged(str)"),
        #     self.__cRecNameLineEdit)
        # self.main.connect(
        #     self.ui.cRecNameLineEdit, SIGNAL("textChanged(str)"),
        #     self.__cRecNameLineEdit)

    # sets the tab order of subframe
    # \param first first widget from the parent dialog
    # \param last last widget from the parent dialog
    def setTabOrder(self, first, last):
        self.main.setTabOrder(first, self.ui.cRecNameLineEdit)
        self.main.setTabOrder(self.ui.cRecNameLineEdit, last)

    # updates datasource ui
    # \param datasource class
    def updateForm(self, datasource):
        if datasource.var['CLIENT'].recordName is not None:
            self.ui.cRecNameLineEdit.setText(
                datasource.var['CLIENT'].recordName)

    # sets the form from the DOM node
    # \param datasource class
    def setFromNode(self, datasource):
        record = self.main.node.firstChildElement(str("record"))
        if record.nodeName() != "record":
            QMessageBox.warning(self.main, "Internal error",
                                "Missing <record> tag")
        else:
            attributeMap = record.attributes()
            datasource.var['CLIENT'].recordName = unicode(
                attributeMap.namedItem("name").nodeValue()
                if attributeMap.contains("name") else "")

    # copies parameters from form to datasource instance
    # \param datasource class
    def fromForm(self, datasource):
        recName = unicode(self.ui.cRecNameLineEdit.text())

        if not recName:
            QMessageBox.warning(self.main, "Empty record name",
                                "Please define the record name")
            self.ui.cRecNameLineEdit.setFocus()
            return
        datasource.var['CLIENT'].recordName = recName

    # creates datasource nodes
    # \param datasource class
    # \param root root node
    # \param elem datasource node
    def createNodes(self, datasource, root, elem):
        record = root.createElement(str("record"))
        record.setAttribute(
            str("name"), str(datasource.var['CLIENT'].recordName))
        elem.appendChild(record)


# DB dialog impementation
class DBSource(object):
    # allowed subitems
    subItems = ["query", "database", "doc"]
    # variables
    var = {
        # database type
        'dbtype': 'MYSQL',
        # database format
        'dataFormat': 'SCALAR',
        # database query
        'query': "",
        # database parameters
        'parameters': {}
    }

    # constructor
    # \param main datasource dialog
    def __init__(self, main):
        # widget class
        self.widgetClass = _dbformclass
        # widget
        self.ui = None
        # main datasource dialog
        self.main = main

        # database parameters
        self.dbParam = {}

        # parameter map for xml tags
        self.__dbmap = {
            "dbname": "DB name",
            "hostname": "DB host",
            "port": "DB port",
            "user": "DB user",
            "passwd": "DB password",
            "mycnf": "Mysql cnf",
            "mode": "Oracle mode"
        }
        self.__idbmap = dict(zip(self.__dbmap.values(), self.__dbmap.keys()))

    # clears widget parameters
    def clear(self):
        self.dbParam = {}

    # checks if widget button should be enable
    # \returns if widget button should be enable
    def isEnable(self):
        return bool(self.ui.dQueryLineEdit.text())

    # calls updateUi when the name text is changing
    def __dQueryLineEdit(self):
        combo = unicode(self.main.ui.typeComboBox.currentText())
        self.main.updateUi(combo)

    # calls updateUi when the name text is changing
    # \param text the edited text
    def __dParamComboBox(self, text):
        param = unicode(text)
        if param == 'DB password':
            QMessageBox.warning(
                self, "Unprotected password",
                "Please note that there is no support for "
                "any password protection")

        self.populateParameters(unicode(text))

    # adds an parameter
    #  \brief It runs the Parameter Dialog and fetches parameter name
    #         and value
    def __addParameter(self):
        name = unicode(self.ui.dParamComboBox.currentText())
        if name not in self.dbParam.keys():
            self.dbParam[name] = ""
        self.populateParameters(name)

    # takes a name of the current parameter
    # \returns name of the current parameter
    def __currentTableParameter(self):
        item = self.ui.dParameterTableWidget.item(
            self.ui.dParameterTableWidget.currentRow(), 0)
        if item is None:
            return None
        return item.data(Qt.UserRole)

    # removes an parameter
    #  \brief It removes the current parameter asking before about it
    def __removeParameter(self):
        param = self.__currentTableParameter()
        if param is None:
            return
        if QMessageBox.question(self, "Parameter - Remove",
                                "Remove parameter: %s = \'%s\'".encode()
                                % (param, self.dbParam[unicode(param)]),
                                QMessageBox.Yes | QMessageBox.No,
                                QMessageBox.Yes) == QMessageBox.No:
            return
        if unicode(param) in self.dbParam.keys():
            self.dbParam.pop(unicode(param))
            self.populateParameters()

    # changes the current value of the parameter
    # \brief It changes the current value of the parameter and informs
    #        the user that parameter names arenot editable
    def __tableItemChanged(self, item):
        param = self.__currentTableParameter()
        if unicode(param) not in self.dbParam.keys():
            return
        column = self.ui.dParameterTableWidget.currentColumn()
        if column == 1:
            self.dbParam[unicode(param)] = unicode(item.text())
        if column == 0:
            QMessageBox.warning(
                self, "Parameter name is not editable",
                "To change the parameter name, "
                "please remove the parameter and add the new one")
        self.populateParameters()

    # fills in the paramter table
    # \param selectedParameter selected parameter
    def populateParameters(self, selectedParameter=None):
        selected = None
        self.ui.dParameterTableWidget.clear()
        self.ui.dParameterTableWidget.setSortingEnabled(False)
        self.ui.dParameterTableWidget.setRowCount(len(self.dbParam))
        headers = ["Name", "Value"]
        self.ui.dParameterTableWidget.setColumnCount(len(headers))
        self.ui.dParameterTableWidget.setHorizontalHeaderLabels(headers)
        for row, name in enumerate(self.dbParam):
            item = QTableWidgetItem(name)
            item.setData(Qt.UserRole, (name))
            self.ui.dParameterTableWidget.setItem(row, 0, item)
            item2 = QTableWidgetItem(self.dbParam[name])
            if selectedParameter is not None and selectedParameter == name:
                selected = item2
            self.ui.dParameterTableWidget.setItem(row, 1, item2)
        self.ui.dParameterTableWidget.setSortingEnabled(True)
        self.ui.dParameterTableWidget.resizeColumnsToContents()
        self.ui.dParameterTableWidget.horizontalHeader()\
            .setStretchLastSection(True)
        if selected is not None:
            selected.setSelected(True)
            self.ui.dParameterTableWidget.setCurrentItem(selected)
            self.ui.dParameterTableWidget.editItem(selected)

    # connects the dialog actions
    def connectWidgets(self):
        try:
            self.ui.dQueryLineEdit.textChanged.disconnect(
                self.__dQueryLineEdit)
        except Exception:
            pass
        try:
            self.ui.dParamComboBox.currentIndexChanged.disconnect(
                self.__dParamComboBox)
        except Exception:
            pass
        try:
            self.ui.dAddPushButton.clicked.disconnect(self.__addParameter)
        except Exception:
            pass
        try:
            self.ui.dRemovePushButton.clicked.disconnect(
                self.__removeParameter)
        except Exception:
            pass
        try:
            self.ui.dParameterTableWidget.itemChanged.disconnect(
                self.__tableItemChanged)
        except Exception:
            pass
        # self.main.disconnect(
        #     self.ui.dQueryLineEdit, SIGNAL("textChanged(str)"),
        #     self.__dQueryLineEdit)
        # self.main.disconnect(
        #     self.ui.dParamComboBox, SIGNAL("currentIndexChanged(str)"),
        #     self.__dParamComboBox)
        # self.main.disconnect(
        #     self.ui.dAddPushButton, SIGNAL("clicked()"), self.__addParameter)
        # self.main.disconnect(
        #     self.ui.dRemovePushButton, SIGNAL("clicked()"),
        #     self.__removeParameter)
        # self.main.disconnect(
        #     self.ui.dParameterTableWidget,
        #     SIGNAL("itemChanged(QTableWidgetItem*)"),
        #     self.__tableItemChanged)

        self.ui.dQueryLineEdit.textChanged.connect(self.__dQueryLineEdit)
        self.ui.dParamComboBox.currentIndexChanged.connect(
            self.__dParamComboBox)
        self.ui.dAddPushButton.clicked.connect(self.__addParameter)
        self.ui.dRemovePushButton.clicked.connect(self.__removeParameter)
        self.ui.dParameterTableWidget.itemChanged.connect(
            self.__tableItemChanged)

        # self.main.connect(
        #     self.ui.dQueryLineEdit,
        #     SIGNAL("textChanged(str)"), self.__dQueryLineEdit)
        # self.main.connect(
        #     self.ui.dParamComboBox, SIGNAL("currentIndexChanged(str)"),
        #     self.__dParamComboBox)
        # self.main.connect(
        #     self.ui.dAddPushButton, SIGNAL("clicked()"), self.__addParameter)
        # self.main.connect(
        #     self.ui.dRemovePushButton, SIGNAL("clicked()"),
        #     self.__removeParameter)
        # self.main.connect(
        #     self.ui.dParameterTableWidget,
        #     SIGNAL("itemChanged(QTableWidgetItem*)"),
        #     self.__tableItemChanged)

    # sets the tab order of subframe
    # \param first first widget from the parent dialog
    # \param last last widget from the parent dialog
    def setTabOrder(self, first, last):
        self.main.setTabOrder(first, self.ui.dTypeComboBox)
        self.main.setTabOrder(
            self.ui.dTypeComboBox, self.ui.dFormatComboBox)
        self.main.setTabOrder(
            self.ui.dFormatComboBox, self.ui.dQueryLineEdit)
        self.main.setTabOrder(
            self.ui.dQueryLineEdit, self.ui.dParamComboBox)
        self.main.setTabOrder(
            self.ui.dParamComboBox, self.ui.dAddPushButton)
        self.main.setTabOrder(
            self.ui.dAddPushButton, self.ui.dRemovePushButton)
        self.main.setTabOrder(
            self.ui.dRemovePushButton, self.ui.dParameterTableWidget)
        self.main.setTabOrder(self.ui.dParameterTableWidget, last)

    # updates datasource ui
    # \param datasource class
    def updateForm(self, datasource):
        if datasource.var['DB'].dbtype is not None:
            index = self.ui.dTypeComboBox.findText(
                unicode(datasource.var['DB'].dbtype))
            if index > -1:
                self.ui.dTypeComboBox.setCurrentIndex(index)
            else:
                datasource.var['DB'].dbtype = 'MYSQL'
        if datasource.var['DB'].dataFormat is not None:
            index = self.ui.dFormatComboBox.findText(
                unicode(datasource.var['DB'].dataFormat))
            if index > -1:
                self.ui.dFormatComboBox.setCurrentIndex(index)
            else:
                datasource.var['DB'].dataFormat = 'SCALAR'

        if datasource.var['DB'].query is not None:
            self.ui.dQueryLineEdit.setText(datasource.var['DB'].query)

        self.dbParam = {}
        for par in datasource.var['DB'].parameters.keys():
            index = self.ui.dParamComboBox.findText(unicode(par))
            if index < 0:
                QMessageBox.warning(
                    self.main, "Unregistered parameter",
                    "Unknown parameter %s = '%s' will be removed."
                    % (par, datasource.var['DB'].parameters[unicode(par)]))
                datasource.var['DB'].parameters.pop(unicode(par))
            else:
                self.dbParam[unicode(par)] = datasource.var['DB'].parameters[
                    (unicode(par))]
        self.populateParameters()

    # sets the form from the DOM node
    # \param datasource class
    def setFromNode(self, datasource):
        database = self.main.node.firstChildElement(str("database"))
        if database.nodeName() != "database":
            QMessageBox.warning(self.main, "Internal error",
                                "Missing <database> tag")
        else:
            attributeMap = database.attributes()

            for i in range(attributeMap.count()):
                name = unicode(attributeMap.item(i).nodeName())
                if name == 'dbtype':
                    datasource.var['DB'].dbtype = unicode(
                        attributeMap.item(i).nodeValue())
                elif name in self.__dbmap:
                    datasource.var['DB'].parameters[self.__dbmap[name]] = \
                        unicode(attributeMap.item(i).nodeValue())
                    self.dbParam[self.__dbmap[name]] = unicode(
                        attributeMap.item(i).nodeValue())

        if not datasource.var['DB'].dbtype:
            datasource.var['DB'].dbtype = 'MYSQL'
        text = unicode(DomTools.getText(database))
        datasource.var['DB'].parameters['Oracle DSN'] = unicode(text).strip() \
            if text else ""
        self.dbParam['Oracle DSN'] = unicode(text).strip() \
            if text else ""

        query = self.main.node.firstChildElement(str("query"))
        if query.nodeName() != "query":
            QMessageBox.warning(self.main, "Internal error",
                                "Missing <query> tag")
        else:
            attributeMap = query.attributes()

            datasource.var['DB'].dataFormat = unicode(
                attributeMap.namedItem("format").nodeValue()
                if attributeMap.contains("format") else "SCALAR")

        text = unicode(DomTools.getText(query))
        datasource.var['DB'].query = unicode(text).strip() if text else ""

    # copies parameters from form to datasource instance
    # \param datasource class
    def fromForm(self, datasource):
        query = unicode(self.ui.dQueryLineEdit.text()).strip()
        if not query:
            QMessageBox.warning(self.main, "Empty query",
                                "Please define the DB query")
            self.ui.dQueryLineEdit.setFocus()
            return
        datasource.var['DB'].query = query
        datasource.var['DB'].dbtype = unicode(
            self.ui.dTypeComboBox.currentText())
        datasource.var['DB'].dataFormat = unicode(
            self.ui.dFormatComboBox.currentText())

        datasource.var['DB'].parameters.clear()
        for par in self.dbParam.keys():
            datasource.var['DB'].parameters[par] = self.dbParam[par]

    # creates datasource nodes
    # \param datasource class
    # \param root root node
    # \param elem datasource node
    def createNodes(self, datasource, root, elem):
        db = root.createElement(str("database"))
        db.setAttribute(
            str("dbtype"), str(datasource.var['DB'].dbtype))
        for par in datasource.var['DB'].parameters.keys():
            if par == 'Oracle DSN':
                newText = root.createTextNode(
                    str(datasource.var['DB'].parameters[par]))
                db.appendChild(newText)
            else:
                db.setAttribute(str(self.__idbmap[par]),
                                str(datasource.var['DB'].parameters[par]))
        elem.appendChild(db)

        query = root.createElement(str("query"))
        query.setAttribute(str("format"),
                           str(datasource.var['DB'].dataFormat))
        if datasource.var['DB'].query:
            newText = root.createTextNode(str(datasource.var['DB'].query))
            query.appendChild(newText)

        elem.appendChild(query)


# TANGO dialog impementation
class TangoSource(object):
    # allowed subitems
    subItems = ["device", "record", "doc"]

    # variables
    var = {
        # Tango device name
        'deviceName': u'',
        # Tango member name
        'memberName': u'',
        # Tango member name
        'memberType': u'',
        # Tango host name
        'host': u'',
        # Tango host name
        'port': u'',
        # encoding for DevEncoded Tango types
        'encoding': u'',
        # group for Tango DataSources
        'group': u''
    }

    # \param main datasource dialog
    def __init__(self, main):
        # widget class
        self.widgetClass = _tangoformclass
        # widget
        self.ui = None
        # main datasource dialog
        self.main = main

    # checks if widget button should be enable
    # \returns if widget button should be enable
    def isEnable(self):
        return bool(self.ui.tDevNameLineEdit.text()) and \
            bool(self.ui.tMemberNameLineEdit.text())

    # sets the tab order of subframe
    # \param first first widget from the parent dialog
    # \param last last widget from the parent dialog
    def setTabOrder(self, first, last):
        self.main.setTabOrder(first, self.ui.tDevNameLineEdit)
        self.main.setTabOrder(
            self.ui.tDevNameLineEdit, self.ui.tMemberComboBox)
        self.main.setTabOrder(
            self.ui.tMemberComboBox, self.ui.tMemberNameLineEdit)
        self.main.setTabOrder(
            self.ui.tMemberNameLineEdit, self.ui.tHostLineEdit)
        self.main.setTabOrder(
            self.ui.tHostLineEdit, self.ui.tPortLineEdit)
        self.main.setTabOrder(
            self.ui.tPortLineEdit, self.ui.tEncodingLineEdit)
        self.main.setTabOrder(
            self.ui.tEncodingLineEdit, self.ui.tGroupLineEdit)
        self.main.setTabOrder(self.ui.tGroupLineEdit, last)

    # updates datasource ui
    # \param datasource class
    def updateForm(self, datasource):
        if datasource.var['TANGO'].deviceName is not None:
            self.ui.tDevNameLineEdit.setText(
                datasource.var['TANGO'].deviceName)
        if datasource.var['TANGO'].memberName is not None:
            self.ui.tMemberNameLineEdit.setText(
                datasource.var['TANGO'].memberName)
        if datasource.var['TANGO'].memberType is not None:
            index = self.ui.tMemberComboBox.findText(
                unicode(datasource.var['TANGO'].memberType))
            if index > -1:
                self.ui.tMemberComboBox.setCurrentIndex(index)
            else:
                datasource.var['TANGO'].memberType = 'attribute'
        if datasource.var['TANGO'].host is not None:
            self.ui.tHostLineEdit.setText(datasource.var['TANGO'].host)
        if datasource.var['TANGO'].port is not None:
            self.ui.tPortLineEdit.setText(datasource.var['TANGO'].port)
        if datasource.var['TANGO'].encoding is not None:
            self.ui.tEncodingLineEdit.setText(datasource.var['TANGO'].encoding)
        if datasource.var['TANGO'].group is not None:
            self.ui.tGroupLineEdit.setText(datasource.var['TANGO'].group)

    # calls updateUi when the name text is changing
    def __tDevNameLineEdit(self):
        combo = unicode(self.main.ui.typeComboBox.currentText())
        self.main.updateUi(combo)

    # calls updateUi when the name text is changing
    def __tMemberNameLineEdit(self):
        combo = unicode(self.main.ui.typeComboBox.currentText())
        self.main.updateUi(combo)

    # connects the dialog actions
    def connectWidgets(self):
        try:
            self.ui.tDevNameLineEdit.textChanged.disconnect(
                self.__tDevNameLineEdit)
        except Exception:
            pass
        try:
            self.ui.tMemberNameLineEdit.textChanged.disconnect(
                self.__tMemberNameLineEdit)
        except Exception:
            pass
        # self.main.disconnect(
        #     self.ui.tDevNameLineEdit,
        #     SIGNAL("textChanged(str)"),
        #     self.__tDevNameLineEdit)
        # self.main.disconnect(
        #     self.ui.tMemberNameLineEdit, SIGNAL("textChanged(str)"),
        #     self.__tMemberNameLineEdit)
        self.ui.tDevNameLineEdit.textChanged.connect(
            self.__tDevNameLineEdit)
        self.ui.tMemberNameLineEdit.textChanged.connect(
            self.__tMemberNameLineEdit)
        # self.main.connect(
        #     self.ui.tDevNameLineEdit, SIGNAL("textChanged(str)"),
        #     self.__tDevNameLineEdit)
        # self.main.connect(
        #     self.ui.tMemberNameLineEdit, SIGNAL("textChanged(str)"),
        #     self.__tMemberNameLineEdit)

    # sets the form from the DOM node
    # \param datasource class
    def setFromNode(self, datasource):
        record = self.main.node.firstChildElement(str("record"))
        if record.nodeName() != "record":
            QMessageBox.warning(self.main, "Internal error",
                                "Missing <record> tag")
        else:
            attributeMap = record.attributes()
            datasource.var['TANGO'].memberName = unicode(
                attributeMap.namedItem("name").nodeValue()
                if attributeMap.contains("name") else "")

        device = self.main.node.firstChildElement(str("device"))
        if device.nodeName() != "device":
            QMessageBox.warning(self.main, "Internal error",
                                "Missing <device> tag")
        else:
            attributeMap = device.attributes()
            datasource.var['TANGO'].deviceName = unicode(
                attributeMap.namedItem("name").nodeValue()
                if attributeMap.contains("name") else "")
            datasource.var['TANGO'].memberType = unicode(
                attributeMap.namedItem("member").nodeValue()
                if attributeMap.contains("member") else "attribute")
            datasource.var['TANGO'].host = unicode(
                attributeMap.namedItem("hostname").nodeValue()
                if attributeMap.contains("hostname") else "")
            datasource.var['TANGO'].port = unicode(
                attributeMap.namedItem("port").nodeValue()
                if attributeMap.contains("port") else "")
            datasource.var['TANGO'].encoding = unicode(
                attributeMap.namedItem("encoding").nodeValue()
                if attributeMap.contains("encoding") else "")
            datasource.var['TANGO'].group = unicode(
                attributeMap.namedItem("group").nodeValue()
                if attributeMap.contains("group") else "")

    # copies parameters from form to datasource instance
    # \param datasource class
    def fromForm(self, datasource):
        devName = unicode(self.ui.tDevNameLineEdit.text())
        memName = unicode(self.ui.tMemberNameLineEdit.text())
        if not devName:
            QMessageBox.warning(self.main, "Empty device name",
                                "Please define the device name")
            self.ui.tDevNameLineEdit.setFocus()
            return
        if not memName:
            QMessageBox.warning(self.main, "Empty member name",
                                "Please define the member name")
            self.ui.tMemberNameLineEdit.setFocus()
            return
        datasource.var['TANGO'].deviceName = devName
        datasource.var['TANGO'].memberName = memName
        datasource.var['TANGO'].memberType = unicode(
            self.ui.tMemberComboBox.currentText())
        datasource.var['TANGO'].host = unicode(self.ui.tHostLineEdit.text())
        datasource.var['TANGO'].port = unicode(self.ui.tPortLineEdit.text())
        datasource.var['TANGO'].encoding = unicode(
            self.ui.tEncodingLineEdit.text())
        datasource.var['TANGO'].group = unicode(self.ui.tGroupLineEdit.text())

    # creates datasource nodes
    # \param datasource class
    # \param root root node
    # \param elem datasource node
    def createNodes(self, datasource, root, elem):
        record = root.createElement(str("record"))
        record.setAttribute(str("name"),
                            str(datasource.var['TANGO'].memberName))
        elem.appendChild(record)

        device = root.createElement(str("device"))
        device.setAttribute(str("name"),
                            str(datasource.var['TANGO'].deviceName))
        device.setAttribute(str("member"),
                            str(datasource.var['TANGO'].memberType))
        if datasource.var['TANGO'].host:
            device.setAttribute(str("hostname"),
                                str(datasource.var['TANGO'].host))
        if datasource.var['TANGO'].port:
            device.setAttribute(str("port"),
                                str(datasource.var['TANGO'].port))
        if datasource.var['TANGO'].encoding:
            device.setAttribute(str("encoding"),
                                str(datasource.var['TANGO'].encoding))
        if datasource.var['TANGO'].group:
            device.setAttribute(str("group"),
                                str(datasource.var['TANGO'].group))
        elem.appendChild(device)


# PYEVAL dialog impementation
class PyEvalSource(object):
    # allowed subitems
    subItems = ["datasource", "result", "doc"]

    # variables
    var = {
        # pyeval result variable
        'result': "ds.result",
        # pyeval datasource variables
        'input': "",
        # pyeval python script
        'script': "",
        # pyeval datasources
        'dataSources': {}
    }

    # \param main datasource dialog
    def __init__(self, main):
        # widget class
        self.widgetClass = _pyevalformclass
        # widget
        self.ui = None
        # main datasource dialog
        self.main = main

    # checks if widget button should be enable
    # \returns if widget button should be enable
    def isEnable(self):
        return True

    # connects the dialog actions
    def connectWidgets(self):
        pass

    # sets the tab order of subframe
    # \param first first widget from the parent dialog
    # \param last last widget from the parent dialog
    def setTabOrder(self, first, last):
        self.main.setTabOrder(first, self.ui.peInputLineEdit)
        self.main.setTabOrder(
            self.ui.peInputLineEdit, self.ui.peScriptTextEdit)
        self.main.setTabOrder(
            self.ui.peScriptTextEdit, self.ui.peResultLineEdit)
        self.main.setTabOrder(self.ui.peResultLineEdit, last)

    # updates datasource ui
    # \param datasource class
    def updateForm(self, datasource):
        if datasource.var['PYEVAL'].result is not None:
            self.ui.peResultLineEdit.setText(datasource.var['PYEVAL'].result)
        if datasource.var['PYEVAL'].input is not None:
            self.ui.peInputLineEdit.setText(datasource.var['PYEVAL'].input)
        if datasource.var['PYEVAL'].script is not None:
            self.ui.peScriptTextEdit.setText(datasource.var['PYEVAL'].script)

    # sets the form from the DOM node
    # \param datasource class
    def setFromNode(self, datasource):
        res = self.main.node.firstChildElement(str("result"))
        text = DomTools.getText(res)
        while len(text) > 0 and text[0] == '\n':
            text = text[1:]
        datasource.var['PYEVAL'].script = unicode(text) if text else ""
        attributeMap = res.attributes()
        datasource.var['PYEVAL'].result = unicode(
            "ds." + attributeMap.namedItem("name").nodeValue()
            if attributeMap.contains("name") else "")

        ds = DomTools.getText(self.main.node)
        dslist = unicode(ds).strip().split() \
            if unicode(ds).strip() else []
        datasource.var['PYEVAL'].dataSources = {}
        child = self.main.node.firstChildElement(
            str("datasource"))
        while not child.isNull():
            if child.nodeName() == 'datasource':
                attributeMap = child.attributes()
                name = unicode(
                    attributeMap.namedItem("name").nodeValue()
                    if attributeMap.contains("name") else "")
                if name.strip():
                    dslist.append(name.strip())
                    doc = QDomDocument()
                    doc.appendChild(
                        doc.importNode(child, True))
                    datasource.var['PYEVAL'].dataSources[name] = unicode(
                        doc.toString(0))
                    child = child.nextSiblingElement("datasource")

        datasource.var['PYEVAL'].input = " ".join(
            "ds." + (d[13:] if (len(d) > 13 and d[:13] == "$datasources.")
                     else d) for d in dslist)

    # copies parameters from form to datasource instance
    # \param datasource class
    def fromForm(self, datasource):
        datasource.var['PYEVAL'].input = \
            unicode(self.ui.peInputLineEdit.text()).strip()
        datasource.var['PYEVAL'].result = \
            unicode(self.ui.peResultLineEdit.text()).strip()
        script = unicode(self.ui.peScriptTextEdit.toPlainText())
        if not script:
            QMessageBox.warning(self.main, "Empty script",
                                "Please define the PyEval script")
            if hasattr(self.ui, "dQueryLineEdit"):
                self.ui.dQueryLineEdit.setFocus()
            return
        datasource.var['PYEVAL'].script = script

    # creates datasource nodes
    # \param datasource class
    # \param root root node
    # \param elem datasource node
    def createNodes(self, datasource, root, elem):
        res = root.createElement(str("result"))
        rn = str(datasource.var['PYEVAL'].result).strip()
        if rn:
            res.setAttribute(
                str("name"),
                str(rn[3:] if (len(rn) > 3 and rn[:3] == 'ds.') else rn))
        if datasource.var['PYEVAL'].script:
            script = root.createTextNode(
                str(
                    datasource.var['PYEVAL'].script if (
                        len(datasource.var['PYEVAL'].script) > 0 and
                        datasource.var['PYEVAL'].script[0] == '\n') else (
                        "\n" + datasource.var['PYEVAL'].script)))
            res.appendChild(script)
        elem.appendChild(res)
        if datasource.var['PYEVAL'].input:
            dslist = unicode(datasource.var['PYEVAL'].input).split()
            newds = ""
            for d in dslist:
                name = d[3:] if (len(d) > 3 and d[:3] == 'ds.') else d
                if name in datasource.var['PYEVAL'].dataSources.keys():
                    document = QDomDocument()
                    if not document.setContent(
                            datasource.var['PYEVAL'].dataSources[name]):
                        raise ValueError("could not parse XML")
                    else:
                        if self.main and hasattr(self.main, "root"):

                            dsnode = DomTools.getFirstElement(
                                document, "datasource")
                            child = root.importNode(dsnode, True)
                            elem.appendChild(child)

                else:
                    newds = "\n ".join([newds, "$datasources." + name])

            newText = root.createTextNode(str(newds))
            elem.appendChild(newText)


if __name__ == "__main__":
    pass



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2017 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# \package nxsconfigtool nexdatas
# \file DataSourceDlg.py
# Data Source dialog class

""" Provides datasource widget """

from PyQt5.QtCore import QModelIndex, pyqtSlot
from PyQt5.QtWidgets import QApplication

from .NodeDlg import NodeDlg
from .DataSources import ClientSource, TangoSource, DBSource, PyEvalSource
from .DataSourceMethods import DataSourceMethods
from PyQt5 import uic

import os


# from .ui.ui_datasourcedlg import Ui_DataSourceDlg
_formclass, _baseclass = uic.loadUiType(
    os.path.join(os.path.dirname(os.path.abspath(__file__)),
                 "ui", "datasourcedlg.ui"))


# available datasources
dsTypes = {'CLIENT': ClientSource,
           'TANGO': TangoSource,
           'DB': DBSource,
           'PYEVAL': PyEvalSource
           }


# load user datasources
# \param dsJSON json string with user datasources
def appendUserDataSource(dsJSON):
    for dk in dsJSON.keys():
        pkl = dsJSON[dk].split(".")
        dec = __import__(".".join(pkl[:-1]),
                         globals(), locals(), pkl[-1])
        dsTypes[dk] = getattr(dec, pkl[-1])


# dialog defining commmon datasource
class CommonDataSourceDlg(NodeDlg):

    # constructor
    # \param datasource instance
    # \param parent patent instance
    def __init__(self, datasource, parent=None):
        super(CommonDataSourceDlg, self).__init__(parent)

        #  datasource instance
        self.datasource = datasource

        # allowed subitems
        self.subItems = []

        # datasource dialog impementations
        self.imp = {}

        # user interface
        self.ui = _formclass()
        # datasource widget
        self.wg = {}

        # QWidget instances
        self.qwg = {}

        for ds in dsTypes.keys():
            self.imp[ds] = dsTypes[ds](self)
            self.subItems.extend(self.imp[ds].subItems)
            self.wg[ds] = self.imp[ds].widgetClass()
            self.imp[ds].ui = self.wg[ds]

    # sets focus on save button
    # \brief It sets focus on save button
    def setSaveFocus(self):
        if self.ui:
            self.ui.savePushButton.setFocus()

    # updates group user interface
    # \brief It sets enable or disable the OK button
    def updateUi(self, text):
        enable = True
        if text in self.imp.keys():
            enable = self.imp[str(text)].isEnable()
        self.ui.applyPushButton.setEnabled(enable)
        self.ui.savePushButton.setEnabled(enable)
        self.ui.storePushButton.setEnabled(enable)
        self.setTabOrder(self.ui.typeComboBox, self.ui.nameLineEdit)
        self.imp[str(text)].setTabOrder(
            self.ui.nameLineEdit, self.ui.savePushButton)
        self.setTabOrder(self.ui.savePushButton, self.ui.storePushButton)
        self.setTabOrder(self.ui.storePushButton, self.ui.closePushButton)
        self.setTabOrder(self.ui.closePushButton, self.ui.applyPushButton)
        self.setTabOrder(self.ui.applyPushButton, self.ui.resetPushButton)
        self.setTabOrder(self.ui.resetPushButton, self.ui.docTextEdit)

    # shows and hides frames according to typeComboBox
    # \param text the edited text
    @pyqtSlot(str)
    def setFrames(self, text):
        for k in self.qwg.keys():
            if text == k:
                self.qwg[k].show()
            else:
                self.qwg[k].hide()

            if hasattr(self.imp[k], "populateParameters"):
                self.imp[k].populateParameters()

        self.updateUi(text)

    # connects the dialog actions
    def connectWidgets(self):
        try:
            self.ui.typeComboBox.currentIndexChanged[str].disconnect(
                self.setFrames)
        except Exception:
            pass
        self.ui.typeComboBox.currentIndexChanged[str].connect(self.setFrames)
        for k in self.imp.keys():
            self.imp[k].connectWidgets()

    # closes the window and cleans the dialog label
    # \param event closing event
    def closeEvent(self, event):
        if hasattr(self.datasource.dialog, "clearDialog"):
            self.datasource.dialog.clearDialog()
        self.datasource.dialog = None
        if hasattr(self.datasource, "clearDialog"):
            self.datasource.clearDialog()
        event.accept()

    # rejects datasource changes
    def reject(self):
        self.parent().close()
        super(CommonDataSourceDlg, self).reject()


# dialog defining separate datasource
class DataSourceDlg(CommonDataSourceDlg):

    # constructor
    # \param parent patent instance
    def __init__(self, parent=None):
        super(DataSourceDlg, self).__init__(None, parent)

        from .DataSource import CommonDataSource

        # datasource data
        self.datasource = CommonDataSource(parent)
        # datasource methods
        self.__methods = DataSourceMethods(self, self.datasource, parent)

    # updates the form
    # \brief updates the form
    def updateForm(self):
        if hasattr(self, "_DataSourceDlg__methods") and self.__methods:
            return self.__methods.updateForm()

    # clears the dialog
    # \brief clears the dialog
    def clearDialog(self):
        if hasattr(self, "_DataSourceDlg__methods") and self.__methods:
            return self.__methods.setDialog(None)

    # updates the node
    # \brief updates the node
    def updateNode(self, index=QModelIndex()):
        if hasattr(self, "_DataSourceDlg__methods") and self.__methods:
            return self.__methods.updateNode(index)

    # creates GUI
    # \brief creates GUI
    def createGUI(self):
        if hasattr(self, "_DataSourceDlg__methods") and self.__methods:
            return self.__methods.createGUI()

    # sets the form from the DOM node
    # \param node DOM node
    def setFromNode(self, node=None):
        if hasattr(self, "_DataSourceDlg__methods") and self.__methods:
            return self.__methods.setFromNode(node)

    # accepts input text strings
    # \brief It copies the parameters and accept the dialog
    def apply(self):
        if hasattr(self, "_DataSourceDlg__methods") and self.__methods:
            return self.__methods.apply()

    # sets the tree mode used in ComponentDlg without save/close buttons
    # \param enable logical variable which dis-/enables mode
    def treeMode(self, enable=True):
        if hasattr(self, "_DataSourceDlg__methods") and self.__methods:
            return self.__methods.treeMode(enable)

    # connects the save action and stores the apply action
    # \param externalApply apply action
    # \param externalSave save action
    # \param externalClose close action
    # \param externalStore store action
    # \param externalDSLink dsource link action
    def connectExternalActions(self, externalApply=None, externalSave=None,
                               externalClose=None, externalStore=None,
                               externalDSLink=None):
        if hasattr(self, "_DataSourceDlg__methods") and self.__methods:
            return self.__methods.connectExternalActions(
                externalApply, externalSave,
                externalClose, externalStore)


if __name__ == "__main__":
    import sys
    from PyQt5.QtGui import QWidget
    # Qt application
    app = QApplication(sys.argv)

    # the second datasource form

    w = QWidget()
    w.show()
    # datasource form
    form2 = DataSourceDlg(w)
    form2.createGUI()
    form2.treeMode(True)

    form2.show()

    app.exec_()



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2017 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# \package nxsconfigtool nexdatas
# \file FieldDlg.py
# Field dialog class

""" field widget """

import copy

from PyQt5.QtWidgets import (QMessageBox, QTableWidgetItem)
from PyQt5.QtCore import (Qt, QModelIndex)
from PyQt5 import uic

import os
import sys

from .AttributeDlg import AttributeDlg
from .DimensionsDlg import DimensionsDlg
from .NodeDlg import NodeDlg
from .DomTools import DomTools

import logging
# message logger
logger = logging.getLogger("nxsdesigner")

_formclass, _baseclass = uic.loadUiType(
    os.path.join(os.path.dirname(os.path.abspath(__file__)),
                 "ui", "fielddlg.ui"))

if sys.version_info > (3,):
    unicode = str


# dialog defining a field tag
class FieldDlg(NodeDlg):

    # constructor
    # \param parent patent instance
    def __init__(self, parent=None):
        super(FieldDlg, self).__init__(parent)

        # field name
        self.name = u''
        # field type
        self.nexusType = u''
        # field units
        self.units = u''
        # field value
        self.value = u''
        # field doc
        self.doc = u''
        # field attributes
        self.attributes = {}
        self.__attributes = {}

        # rank
        self.rank = 0
        # dimensions
        self.dimensions = []
        self.__dimensions = []

        # allowed subitems
        self.subItems = ["attribute", "datasource", "doc", "dimensions",
                         "enumeration", "strategy"]

        # user interface
        self.ui = _formclass()

    # provides the state of the field dialog
    # \returns state of the field in tuple
    def getState(self):
        attributes = copy.copy(self.attributes)
        dimensions = copy.copy(self.dimensions)

        state = (self.name,
                 self.nexusType,
                 self.units,
                 self.value,
                 self.doc,
                 self.rank,
                 attributes,
                 dimensions
                 )
        return state

    # sets the state of the field dialog
    # \param state field state written in tuple
    def setState(self, state):

        (self.name,
         self.nexusType,
         self.units,
         self.value,
         self.doc,
         self.rank,
         attributes,
         dimensions
         ) = state
        self.attributes = copy.copy(attributes)
        self.dimensions = copy.copy(dimensions)

    # links dataSource
    # \param dsName datasource name
    def linkDataSource(self, dsName):
        self.value = "$%s.%s" % (self.dsLabel, dsName)
        self.updateForm()

    # updates the field dialog
    # \brief It sets the form local variables
    def updateForm(self):
        if self.name is not None:
            self.ui.nameLineEdit.setText(self.name)
        if self.nexusType is not None:
            index = self.ui.typeComboBox.findText(unicode(self.nexusType))
            if index > -1:
                self.ui.typeComboBox.setCurrentIndex(index)
                self.ui.otherFrame.hide()
            else:
                index2 = self.ui.typeComboBox.findText('other ...')
                self.ui.typeComboBox.setCurrentIndex(index2)
                self.ui.typeLineEdit.setText(self.nexusType)
                self.ui.otherFrame.show()
        else:
            index = self.ui.typeComboBox.findText(unicode("None"))
            self.ui.typeComboBox.setCurrentIndex(index)
            self.ui.otherFrame.hide()
        if self.doc is not None:
            self.ui.docTextEdit.setText(self.doc)
        if self.units is not None:
            self.ui.unitsLineEdit.setText(self.units)
        if self.value is not None:
            self.ui.valueLineEdit.setText(self.value)

        if self.rank < len(self.dimensions):
            self.rank = len(self.dimensions)

        if self.dimensions:
            label = self.dimensions.__str__()
            self.ui.dimLabel.setText("%s" % label.replace('None', '*'))
        elif self.rank > 0:
            label = ([None] * (self.rank)).__str__()
            self.ui.dimLabel.setText("%s" % label.replace('None', '*'))
        else:
            self.ui.dimLabel.setText("[]")

        self.__dimensions = []
        for dm in self.dimensions:
            self.__dimensions.append(dm)

        self.__attributes.clear()
        for at in self.attributes.keys():
            self.__attributes[unicode(at)] = self.attributes[(unicode(at))]

        self.populateAttributes()

    #  creates GUI
    # \brief It calls setupUi and  connects signals and slots
    def createGUI(self):
        self.ui.setupUi(self)

        self.updateForm()

        self.__updateUi()

        self.ui.resetPushButton.clicked.connect(self.reset)
        self.ui.attributeTableWidget.itemChanged.connect(
            self.__tableItemChanged)
        self.ui.addPushButton.clicked.connect(
            self.__addAttribute)
        self.ui.removePushButton.clicked.connect(
            self.__removeAttribute)
        self.ui.dimPushButton.clicked.connect(
            self.__changeDimensions)

        self.ui.nameLineEdit.textEdited[str].connect(
            self.__updateUi)
        self.ui.typeComboBox.currentIndexChanged[str].connect(
            self.__currentIndexChanged)

        self.populateAttributes()

    # sets the form from the DOM node
    # \param node DOM node
    def setFromNode(self, node=None):
        if node:
            # defined in NodeDlg
            self.node = node
        if not self.node:
            return
        attributeMap = self.node.attributes()

        self.name = unicode(attributeMap.namedItem("name").nodeValue()
                            if attributeMap.contains("name") else "")
        self.nexusType = unicode(attributeMap.namedItem("type").nodeValue()
                                 if attributeMap.contains("type") else "")
        self.units = unicode(attributeMap.namedItem("units").nodeValue()
                             if attributeMap.contains("units") else "")

        text = DomTools.getText(self.node)
        self.value = unicode(text).strip() if text else ""

        self.attributes.clear()
        self.__attributes.clear()
        for i in range(attributeMap.count()):
            attribute = attributeMap.item(i)
            attrName = unicode(attribute.nodeName())
            if attrName != "name" and attrName != "type" \
                    and attrName != "units":
                self.attributes[attrName] = unicode(attribute.nodeValue())
                self.__attributes[attrName] = unicode(attribute.nodeValue())

        dimens = self.node.firstChildElement(str("dimensions"))
        attributeMap = dimens.attributes()

        self.dimensions = []
        self.__dimensions = []
        if attributeMap.contains("rank"):
            try:
                self.rank = int(attributeMap.namedItem("rank").nodeValue())
                if self.rank < 0:
                    self.rank = 0
            except Exception:
                self.rank = 0
        else:
            self.rank = 0
        if self.rank > 0:
            child = dimens.firstChild()
            while not child.isNull():
                if child.isElement() and child.nodeName() == "dim":
                    attributeMap = child.attributes()
                    index = None
                    value = None
                    try:
                        if attributeMap.contains("index"):
                            index = int(
                                attributeMap.namedItem("index").nodeValue())
                        if attributeMap.contains("value"):
                            value = str(
                                attributeMap.namedItem("value").nodeValue())
                    except Exception:
                        pass

                    text = DomTools.getText(child)
                    if text and "$datasources." in text:
                        value = str(text).strip()
                    if index < 1:
                        index = None
                    if index is not None:
                        while len(self.dimensions) < index:
                            self.dimensions.append(None)
                            self.__dimensions.append(None)
                        self.__dimensions[index - 1] = value
                        self.dimensions[index - 1] = value

                child = child.nextSibling()

        if self.rank < len(self.dimensions):
            self.rank = len(self.dimensions)
            self.rank = len(self.__dimensions)
        elif self.rank > len(self.dimensions):
            self.dimensions.extend(
                [None] * (self.rank - len(self.dimensions)))
            self.__dimensions.extend(
                [None] * (self.rank - len(self.__dimensions)))

        doc = self.node.firstChildElement(str("doc"))
        text = DomTools.getText(doc)
        self.doc = unicode(text).strip() if text else ""

    # adds an attribute
    #  \brief It runs the Field Dialog and fetches attribute name and value
    def __addAttribute(self):
        aform = AttributeDlg()
        if aform.exec_():

            if aform.name not in self.__attributes.keys():
                self.__attributes[aform.name] = aform.value
                self.populateAttributes(aform.name)
            else:
                QMessageBox.warning(
                    self, "Attribute name exists",
                    "To change the attribute value, "
                    "please edit the value in the attribute table")

    # changing dimensions of the field
    #  \brief It runs the Dimensions Dialog and fetches rank
    #         and dimensions from it
    def __changeDimensions(self):
        dform = DimensionsDlg(self)
        dform.rank = self.rank
        dform.lengths = [ln for ln in self.__dimensions]
        dform.createGUI()
        if dform.exec_():
            self.rank = dform.rank
            if self.rank:
                self.__dimensions = [dm for dm in dform.lengths]
            else:
                self.__dimensions = []
            label = self.__dimensions.__str__()
            self.ui.dimLabel.setText("%s" % label.replace('None', '*'))

    # takes a name of the current attribute
    # \returns name of the current attribute
    def __currentTableAttribute(self):
        item = self.ui.attributeTableWidget.item(
            self.ui.attributeTableWidget.currentRow(), 0)
        if item is None:
            return None
        return item.data(Qt.UserRole)

    # removes an attribute
    #  \brief It removes the current attribute asking before about it
    def __removeAttribute(self):
        attr = self.__currentTableAttribute()
        if attr is None:
            return
        if unicode(attr) in self.__attributes.keys():
            self.__attributes.pop(unicode(attr))
            self.populateAttributes()

    # changes the current value of the attribute
    # \brief It changes the current value of the attribute and informs
    #        the user that attribute names arenot editable
    def __tableItemChanged(self, item):
        attr = self.__currentTableAttribute()
        if unicode(attr) not in self.__attributes.keys():
            return
        column = self.ui.attributeTableWidget.currentColumn()
        if column == 1:
            self.__attributes[unicode(attr)] = unicode(item.text())
        if column == 0:
            QMessageBox.warning(
                self, "Attribute name is not editable",
                "To change the attribute name, "
                "please remove the attribute and add the new one")
        self.populateAttributes()

    # fills in the attribute table
    # \param selectedAttribute selected attribute
    def populateAttributes(self, selectedAttribute=None):
        selected = None
        self.ui.attributeTableWidget.clear()
        self.ui.attributeTableWidget.setSortingEnabled(False)
        self.ui.attributeTableWidget.setRowCount(len(self.__attributes))
        headers = ["Name", "Value"]
        self.ui.attributeTableWidget.setColumnCount(len(headers))
        self.ui.attributeTableWidget.setHorizontalHeaderLabels(headers)
        for row, name in enumerate(self.__attributes):
            item = QTableWidgetItem(name)
            item.setData(Qt.UserRole, (name))
            self.ui.attributeTableWidget.setItem(row, 0, item)
            item2 = QTableWidgetItem(self.__attributes[name])
            self.ui.attributeTableWidget.setItem(row, 1, item2)
            if selectedAttribute is not None and selectedAttribute == name:
                selected = item2
        self.ui.attributeTableWidget.setSortingEnabled(True)
        self.ui.attributeTableWidget.resizeColumnsToContents()
        self.ui.attributeTableWidget.horizontalHeader().\
            setStretchLastSection(True)
        if selected is not None:
            selected.setSelected(True)
            self.ui.attributeTableWidget.setCurrentItem(selected)

    # calls updateUi when the name text is changing
    # \param text the edited text
    def __currentIndexChanged(self, text):
        if text == 'other ...':
            self.ui.otherFrame.show()
            self.ui.typeLineEdit.setFocus()
        else:
            self.ui.otherFrame.hide()

    # updates field user interface
    # \brief It sets enable or disable the OK button
    def __updateUi(self):
        enable = bool(self.ui.nameLineEdit.text())
        self.ui.applyPushButton.setEnabled(enable)

    # appends newElement
    # \param newElement DOM node to append
    # \param parent parent DOM node
    def appendElement(self, newElement, parent):
        singles = {"datasource": "DataSource", "strategy": "Strategy"}
        if unicode(newElement.nodeName()) in singles:
            if not self.node:
                return
            child = self.node.firstChild()
            while not child.isNull():
                if child.nodeName() == unicode(newElement.nodeName()):
                    QMessageBox.warning(
                        self,
                        "%s exists" % singles[str(newElement.nodeName())],
                        "To add a new %s please remove the old one"
                        % newElement.nodeName())
                    return False
                child = child.nextSibling()

        return NodeDlg.appendElement(self, newElement, parent)

    # applys input text strings
    # \brief It copies the field name and type from lineEdit widgets
    #        and apply the dialog
    def apply(self):
        self.name = unicode(self.ui.nameLineEdit.text())
        self.units = unicode(self.ui.unitsLineEdit.text())
        self.value = unicode(self.ui.valueLineEdit.text())

        self.nexusType = unicode(self.ui.typeComboBox.currentText())
        if self.nexusType == 'other ...':
            self.nexusType = unicode(self.ui.typeLineEdit.text())
        elif self.nexusType == 'None':
            self.nexusType = u''

        self.doc = unicode(self.ui.docTextEdit.toPlainText())

        index = self.view.currentIndex()
        finalIndex = self.view.model().createIndex(
            index.row(), 2, index.parent().internalPointer())

        self.view.expand(index)

        self.attributes.clear()
        for at in self.__attributes.keys():
            self.attributes[at] = self.__attributes[at]

        self.dimensions = []
        for dm in self.__dimensions:
            self.dimensions.append(dm)

        if self.node and self.root and self.node.isElement():
            self.updateNode(index)

        if index.column() != 0:
            index = self.view.model().index(index.row(), 0, index.parent())
        self.view.expand(index)
        self.view.model().dataChanged.emit(index, finalIndex)
        self.view.model().dataChanged.emit(index, index)

    # updates the Node
    # \brief It sets node from the dialog variables
    def updateNode(self, index=QModelIndex()):
        elem = self.node.toElement()

        mindex = self.view.currentIndex() if not index.isValid() else index

        attributeMap = self.node.attributes()
        for i in range(attributeMap.count()):
            attributeMap.removeNamedItem(attributeMap.item(0).nodeName())
        if self.name:
            elem.setAttribute(str("name"), str(self.name))
        if self.nexusType:
            elem.setAttribute(str("type"), str(self.nexusType))
        if self.units:
            elem.setAttribute(str("units"), str(self.units))

        self.replaceText(mindex, unicode(self.value))

        for attr in self.attributes.keys():
            elem.setAttribute(str(attr), str(self.attributes[attr]))

        doc = self.node.firstChildElement(str("doc"))
        if not self.doc and doc and doc.nodeName() == "doc":
            self.removeElement(doc, mindex)
        elif self.doc:
            newDoc = self.root.createElement(str("doc"))
            newText = self.root.createTextNode(str(self.doc))
            newDoc.appendChild(newText)
            if doc and doc.nodeName() == "doc":
                self.replaceElement(doc, newDoc, mindex)
            else:
                self.appendElement(newDoc, mindex)

        dimens = self.node.firstChildElement(str("dimensions"))
        if not self.dimensions and dimens \
                and dimens.nodeName() == "dimensions":
            self.removeElement(dimens, mindex)
        elif self.dimensions:
            newDimens = self.root.createElement(str("dimensions"))
            newDimens.setAttribute(str("rank"),
                                   str(unicode(self.rank)))
            dimDefined = True
            for i in range(min(self.rank, len(self.dimensions))):
                if self.dimensions[i] is None:
                    dimDefined = False
            if dimDefined:
                for i in range(min(self.rank, len(self.dimensions))):
                    dim = self.root.createElement(str("dim"))
                    dim.setAttribute(str("index"), str(unicode(i + 1)))
                    if "$datasources." not in unicode(self.dimensions[i]):

                        dim.setAttribute(str("value"),
                                         str(unicode(self.dimensions[i])))
                    else:
                        dsText = self.root.createTextNode(
                            str(unicode(self.dimensions[i])))
                        dstrategy = self.root.createElement(
                            str("strategy"))
                        dstrategy.setAttribute(
                            str("mode"),
                            str(unicode("CONFIG")))
                        dim.appendChild(dsText)
                        dim.appendChild(dstrategy)

                    newDimens.appendChild(dim)

            if dimens and dimens.nodeName() == "dimensions":
                self.replaceElement(dimens, newDimens, mindex)
            else:
                self.appendElement(newDimens, mindex)


if __name__ == "__main__":
    import sys
    from PyQt5.QtGui import QApplication

    logging.basicConfig(level=logging.DEBUG)

    # Qt application
    app = QApplication(sys.argv)
    # field form
    form = FieldDlg()
    form.name = 'distance'
    form.nexusType = 'NX_FLOAT'
    form.units = 'cm'
    form.attributes = {"signal": "1",
                       "long_name": "source detector distance",
                       "interpretation": "spectrum"}
    form.doc = """Distance between the source and the mca detector.
It should be defined by client."""
    form.dimensions = [3]
    form.value = "1.23,3.43,4.23"
    form.createGUI()
    form.show()
    app.exec_()
    if form.name:
        logger.info("Field: name = \'%s\'" % (form.name))
    if form.nexusType:
        logger.info("       type = \'%s\'" % (form.nexusType))
    if form.units:
        logger.info("       units = \'%s\'" % (form.units))
    if form.attributes:
        logger.info("Other attributes:")
        for k in form.attributes.keys():
            logger.info(" %s = '%s' " % (k, form.attributes[k]))
    if form.value:
        logger.info("Value:\n \'%s\'" % (form.value))
    if form.rank:
        logger.info(" rank = %s" % (form.rank))
    if form.dimensions:
        logger.info("Dimensions:")
        for mrow, mln in enumerate(form.dimensions):
            logger.info(" %s: %s " % (mrow + 1, mln))

    if form.doc:
        logger.info("Doc: \n%s" % form.doc)
